<?php
/*
Plugin Name: RS Buddypress Activity Refresh
PLugin URI: http://buddypress.org/community/groups/rs-buddypress-activity-refresh/
Description: This plugin automatically refresh the Buddypress activity stream
Author: Florian Koenig-Heidinger
Requires at least: 2.9.1 / 1.2.4
Tested up to: 3.2.1 / 1.2.9
Tags: buddypress
Version: 1.0.7.1
Author URI: http://buddypress.org/community/members/Spitzohr/
*/

/**
 * class rsBuddypressActivityRefresh
 */
class rsBuddypressActivityRefresh
{
	/**
	 * Plugin Folder
	 * @access protected
	 * @var string
	 */
	protected $plugin_folder = null;

	/**
	 * Plugin Directory
	 * @access protected
	 * @var string
	 */
	protected $plugin_dir = null;

	/**
	 * Plugin Url
	 * @access protected
	 * @var string
	 */
	protected $plugin_url = null;

	/**
	 * Plugin File
	 * @access protected
	 * @var string
	 */
	protected $plugin_file = null;

	/**
	 * Default Refresh Rate in seconds
	 * @access protected
	 * @var integer
	 */
	protected $defaultRefreshRate = 10;

	/**
	 * Admin Options Name
	 * @access protected
	 * @var string
	 */
	protected $adminOptionsName = 'rsBuddypressActivityRefresh';

	/**
	 * Options array
	 * @access protected
	 * @var array
	 */
	protected $options = null;

	/**
	 * Constructor, starts init method
	 * @access public
	 */
	public function __construct()
	{
	} // function construct()

	/**
	 * Initialising
	 * @access public
	 */
	public function init()
	{
		$this->plugin_folder = substr(dirname(__FILE__), strrpos(dirname(__FILE__), DIRECTORY_SEPARATOR) + 1);
		$this->plugin_url = WP_PLUGIN_URL . '/' . $this->plugin_folder;
		$this->plugin_dir = WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . $this->plugin_folder;
		$this->plugin_file = $this->plugin_folder . '/' . basename(__FILE__);

		$this->getOptions();

		// only add the JavaScript, if the refreshRate > 0
		if ($this->options['refreshRate'] > 0)
		{
			// adding the refresh javascript file
			wp_enqueue_script( 'rs-bp-activity-refresh-ajax-js', $this->plugin_url . '/refresh.js', array( 'jquery' ) );

			// adding JavaScript Refresh Rate to html head
			add_action( 'wp_head', array(&$this, 'addJavaScriptRefreshRate'));

			// add method ajaxRefresh to action hook wp_ajax_rs_bp_activity_refresh
			add_action('wp_ajax_rs_bp_activity_refresh', array(&$this, 'ajaxRefresh'));
		}

		add_action('admin_menu', array(&$this, 'addAdminMenu'));
		add_filter('plugin_action_links', array(&$this, 'addPluginActionLink'), 10, 2 );
	} // function init()

	/**
	 * getOptions()
	 * Load and update options
	 * @access protected
	 */
	protected function getOptions()
	{
		// set default options
		$this->options = array(
			'refreshRate' => $this->defaultRefreshRate,
		);

		// load options
		$savedOptions = get_option($this->adminOptionsName);

		if (!empty($savedOptions))
		{
			foreach ($savedOptions as $key => $option)
			{
				$this->options[$key] = $option;
			}
		}

		// save options
		update_option($this->adminOptionsName, $this->options);

		return $this->options;
	} // getOptions()

	/**
	 * validate and update options ()
	 * Load and update options
	 * @var $options array
	 * @access protected
	 */
	protected function saveOptions( array $options )
	{

		if (!empty($options))
		{
			foreach ($options as $key => $option)
			{
				switch ($key)
				{
					case 'refreshRate':
						if (is_numeric($option))
						{
							$this->options[$key] = (int)$option;
						}
						break;
	
					default:
						$this->options[$key] = $option;
						break;
				}
			}
		}

		// save options
		update_option($this->adminOptionsName, $this->options);
	} // saveOptions()

	/**
	 * add JavaScript variable refreshRate
	 * action hook: wp_head
	 * @access public
	 */
	public function addJavaScriptRefreshRate()
	{
		echo '<script type="text/javascript">var rsActivityRefreshRate = ' . $this->options['refreshRate'] . ';</script>';
	}

	/**
	 * method to answer the ajax request
	 * action hook: wp_ajax_rs_bp_activity_refresh
	 * @access public
	 */
	public function ajaxRefresh()
	{
		// get existing IDs
		$existing_ids = array();
		if (isset($_POST['existing_ids']))
		{
			$existing_ids = explode(',',$_POST['existing_ids']);

			// start the Loop
			// show new comments in stream format
			if (bp_has_activities( bp_ajax_querystring( 'activity' )  . '&display_comments=stream' ))
			{
				while ( bp_activities() )
				{
					bp_the_activity();
					if (
						!in_array(bp_get_activity_id(), $existing_ids) // check if ID doesn't exist yet
						 && (strtotime(bp_get_activity_date_recorded()) >= time() - (5 * 60) ) // get only records from the last 5 minutes
					)
					{
						include( locate_template( array( 'activity/entry.php' ), false ) );
					}
				}
			}
		}
	} // function ajaxRefresh()

	/**
	 * method to add the admin menu
	 * action hook: admin_menu
	 * @access public
	 */
	public function addAdminMenu()
	{
		add_options_page('RS Buddypress Activity Refresh', 'RS Buddypress Activity Refresh', 'manage_options', basename(__FILE__), array(&$this, 'printAdminPage'));
	} // end function addAdminMenu()

	/**
	 * prints out the admin page
	 * @access public
	 */
	public function printAdminPage()
	{
		if (!current_user_can('manage_options'))
		{
			wp_die( __('You do not have sufficient permissions to access this page.') );
		}
										
		if (isset($_POST['update_rsbpactivityrefresh']))
		{
			if (isset($_POST['rsbpactivityrefresh_refreshrate'])) {
				$options['refreshRate'] = $_POST['rsbpactivityrefresh_refreshrate'];
			}	
			$this->saveOptions($options);
			?>
				<div class="updated"><p><strong><?php _e("Settings Updated.", "rs-buddypress-activity-refresh");?></strong></p></div>
			<?php
		}

		?>
			<div class="wrap">
				<form method="post" action="">
					<h2>RS Buddypress Activity Refresh</h2>
					<h3>Refresh Rate</h3>

					<p>This value is the refresh rate in seconds. Default value is 10. If you set this value to 0, the refresh is disabled.</p>
					<p><label for="rsbpactivityrefresh_refreshrate"><input type="text" id="rsbpactivityrefresh_refreshrate" name="rsbpactivityrefresh_refreshrate" value="<?php echo $this->options['refreshRate']; ?>" /> seconds</label></p>

					<div class="submit">
						<input type="submit" name="update_rsbpactivityrefresh" value="<?php _e('Update Settings', 'rs-buddypress-activity-refresh') ?>" />
					</div>
				</form>
 			</div>
		<?php
	}//End function printAdminPage()

	/**
	 * addPluginActionLink
	 * Stolen from Welcome Pack - thanks, Paul!
	 * filter hook: plugin_action_links
	 * @access public
	 * @param array $links
	 * @param string $file
	 */
	public function addPluginActionLink ($links, $file)
	{
		if ($this->plugin_file != $file)
		{
			return $links;
		}
		$settings_link = '<a href="' . admin_url('admin.php?page=' . basename(__FILE__)) . '">' . __( 'Settings', 'rs-buddypress-activity-refresh' ) . '</a>';

		array_unshift( $links, $settings_link );
		return $links;
	} // addPluginActionLink()

} // class rsBuddypressActivityRefresh

/**
 * rsBuddypressActivityRefreshInit()
 *
 * Only load the plugin code if BuddyPress is activated.
 */
function rsBuddypressActivityRefreshInit()
{
	$rsBuddypressActivityRefresh = new rsBuddypressActivityRefresh();
	$rsBuddypressActivityRefresh->init();
}
add_action('bp_init', 'rsBuddypressActivityRefreshInit');

?>